using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace Frief
{

    public class BDSMTwoPillars: MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Frief";
        public string pluginName = "BDSM-Two Pillars";
        public string pluginVersion = "2";
        public string pluginDate = "08/05" + "/2023";
        public string pluginDescription = @"Used";
        #endregion

        #region Vars    

        public JSONStorableBool SawEnabled;

        public GameObject player;
        public GameObject Ks_Fridge;

        protected JSONStorableFloat ScrubA;
        protected JSONStorableFloat ScrubB;
        protected JSONStorableFloat ScrubC;
        protected JSONStorableFloat ScrubD;

        private Vector3 BDSMPillar_UpRightStartPosition;
        private Vector3 BDSMPillar_UpLeftStartPosition;
        private Vector3 BDSMPillar_DownRightStartPosition;
        private Vector3 BDSMPillar_DownLeftStartPosition;

        private float previousValue1;
        private float previousValue2;
        private float previousValue3;
        private float previousValue4;

        private List<Atom> emptyNodes = new List<Atom>();


        private float midY;
        private bool isCoroutineFinished = false;

        private string objectNameSearchString = "Pillars";
        public Transform ObjectRoot;
        public Transform BDSMPillar_UP;
        public Transform BDSMPillar_DOWN;
        public Transform BDSMPillar_UpRight;
        public Transform BDSMPillar_UpLeft;
        public Transform BDSMPillar_DownRight;
        public Transform BDSMPillar_DownLeft;



        private Renderer BDSMPillar_UpRightRenderer;


        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 3600; //3600 for one minute

        private static Atom currentAtom;
        private static string currentAtomName = "None";
        private static string tempAtomName = "None";


        const string atomType = "Sphere";
        #endregion



        public override void Init()
        {

            

            try
            {
                
                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("BDSM-Pillars: This CUA could not be initialized properly.Please select a compatible object and reload the script.");
                    return;
                }

                //**************  ScrubA


                ScrubA = new JSONStorableFloat("UpLeft", 0f, -0.9f, 0.08f, true);
                RegisterFloat(ScrubA);
                CreateSlider(ScrubA, false);
                


                //**************  ScrubB
               

                ScrubB = new JSONStorableFloat("Upright", 0f, -0.9f, 0.08f, true);
                RegisterFloat(ScrubB);
                CreateSlider(ScrubB, false);


                //**************  ScrubC
                ScrubC = new JSONStorableFloat("DownRight", 0f, -0.04f, 0.8f, true);
                RegisterFloat(ScrubC);
                CreateSlider(ScrubC, false);

                //**************  ScrubD
                ScrubD = new JSONStorableFloat("DownLeft", 0f, -0.04f, 0.8f, true);
                RegisterFloat(ScrubD);
                CreateSlider(ScrubD, false);


                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("Help",
                    "<color=#000><size=35><b>Help</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Simple script to allow you to adjust holdingpoints on the twoPillars .\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 1100.0f;


            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
                
          

        }

        private void Start()
        {
            // Hier wird der Code ausgefhrt, nachdem die Szene geladen wurde

            // ...

            if (initLoops < maxInitLoops && editorInitialized == false)
            {
                initLoops++;
                SuperController.LogMessage("start editor");
                StartInitEditor();

            }

            if (initLoops == maxInitLoops)
            {
                SuperController.LogError("This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }

 
        }

        private void StartInitEditor()
        {
            StartCoroutine(InitEditorCoroutine());
        }

        void Destroy()
        {
            RemoveExistingNodes();
        }


        // Update is called with each rendered frame by Unity
        void Update()

        {
            var Slider1 = ScrubA.val;
            var Slider2 = ScrubB.val;
            var Slider3 = ScrubC.val;
            var Slider4 = ScrubD.val;

          

            if (previousValue1 != Slider1)
            {
                Slider1Changed();
            }

            if (previousValue2 != Slider2)
            {
                Slider2Changed();
            }

            if (previousValue3 != Slider3)
            {
                Slider3Changed();
            }

            if (previousValue4 != Slider4)
            {
                Slider4Changed();
            }


            
        }


        private IEnumerator InitEditorCoroutine()
        {

            yield return new WaitForSeconds(5f); // Beispielhafter Wartezeitraum

            ObjectRoot = getObjectRoot(containingAtom.reParentObject);
            if (ObjectRoot == null)
            {
                yield return false;
            }

            BDSMPillar_UP = ObjectRoot.Find("UP");
            BDSMPillar_DOWN = ObjectRoot.Find("Down");
            BDSMPillar_UpLeft = ObjectRoot.Find("UP/Hook_Ur");
            BDSMPillar_UpRight = ObjectRoot.Find("UP/Hook_Ul");
            BDSMPillar_DownRight = ObjectRoot.Find("Down/Hook_Dr");
            BDSMPillar_DownLeft = ObjectRoot.Find("Down/Hook_Dl");
            

            // Warte, bis die Coroutine abgeschlossen ist
            

            if (BDSMPillar_UpLeft != null && BDSMPillar_UpRight != null )
            {
                // Speichern der Startpositionen
                BDSMPillar_UpRightStartPosition = BDSMPillar_UpRight.localPosition;
                BDSMPillar_UpLeftStartPosition = BDSMPillar_UpLeft.localPosition;
                // Speichern der Startpositionen
                BDSMPillar_DownRightStartPosition = BDSMPillar_DownRight.localPosition;
                BDSMPillar_DownLeftStartPosition = BDSMPillar_DownLeft.localPosition;

                ScrubC.defaultVal = BDSMPillar_DownRightStartPosition.y;
                ScrubD.defaultVal = BDSMPillar_DownLeftStartPosition.y;
                ScrubA.defaultVal = BDSMPillar_UpRightStartPosition.y;

                StartCoroutine(SetupEmptyNodes());

                editorInitialized = true;
                if (editorInitialized)
                {
                    yield return true;
                }
            }
            else
            {
                SuperController.LogError("This CUA could not be initialized properly. BDSMPillar_UpLeft or BDSMPillar_UpRight not found");
                yield return false;
            }

            yield return true;
        }



        IEnumerator SetupControlEmptyObject(string name, Vector3 position)
        {
            Atom emptyObject = SuperController.singleton.GetAtomByUid(name);
            if (emptyObject)
            {
                SuperController.singleton.RemoveAtom(emptyObject);
                emptyObject = null;
            }

            if (!emptyObject)
            {
                yield return SuperController.singleton.AddAtomByType("Empty", name);

                emptyObject = SuperController.singleton.GetAtomByUid(name);
                if (emptyObject != null)
                {
                    emptyObject.parentAtom = containingAtom;
                    emptyObject.mainController.transform.position = position;
                }
            }
        }

        IEnumerator SetupEmptyNodes()
        {
            // Finde die Transform-Objekte der BDSMPillar-Objekte
            Transform BDSMPillar_UpLeft = ObjectRoot.Find("UP/Hook_Ur");
            Transform BDSMPillar_UpRight = ObjectRoot.Find("UP/Hook_Ul");
            Transform BDSMPillar_DownRight = ObjectRoot.Find("Down/Hook_Dr");
            Transform BDSMPillar_DownLeft = ObjectRoot.Find("Down/Hook_Dl");

            // berprfe und fge die Empty Nodes hinzu oder fhre das Parenting durch
            Atom Node_UpLeft = SuperController.singleton.GetAtomByUid("Node_UpLeft");
            if (Node_UpLeft == null)
            {
                yield return SetupControlEmptyObject("Node_UpLeft", BDSMPillar_UpLeft.position);
                Node_UpLeft = SuperController.singleton.GetAtomByUid("Node_UpLeft");
                Node_UpLeft.mainController.transform.parent = BDSMPillar_UpLeft;
                emptyNodes.Add(Node_UpLeft);
                
            }
            else
            {
                Node_UpLeft.mainController.transform.parent = BDSMPillar_UpLeft;
            }

            Atom Node_UpRight = SuperController.singleton.GetAtomByUid("Node_UpRight");
            if (Node_UpRight == null)
            {
                yield return SetupControlEmptyObject("Node_UpRight", BDSMPillar_UpRight.position);
                Node_UpRight = SuperController.singleton.GetAtomByUid("Node_UpRight");
                emptyNodes.Add(Node_UpRight);
                Node_UpRight.mainController.transform.parent = BDSMPillar_UpRight;
            }
            else
            {
                Node_UpRight.mainController.transform.parent = BDSMPillar_UpRight;
            }

            Atom Node_DownRight = SuperController.singleton.GetAtomByUid("Node_DownRight");
            if (Node_DownRight == null)
            {
                yield return SetupControlEmptyObject("Node_DownRight", BDSMPillar_DownRight.position);
                Node_DownRight = SuperController.singleton.GetAtomByUid("Node_DownRight");
                Node_DownRight.mainController.transform.parent = BDSMPillar_DownRight;
                emptyNodes.Add(Node_DownRight);
               
            }
            else
            {
                Node_DownRight.mainController.transform.parent = BDSMPillar_DownRight;
            }

            Atom Node_DownLeft = SuperController.singleton.GetAtomByUid("Node_DownLeft");
            if (Node_DownLeft == null)
            {
                yield return SetupControlEmptyObject("Node_DownLeft", BDSMPillar_DownLeft.position);
                Node_DownLeft = SuperController.singleton.GetAtomByUid("Node_DownLeft");
                Node_DownLeft.mainController.transform.parent = BDSMPillar_DownLeft;
                emptyNodes.Add(Node_DownLeft);
                
            }
            else
            {
                Node_DownLeft.mainController.transform.parent = BDSMPillar_DownLeft;
            }
            isCoroutineFinished = true;
            yield return null;

        }


        void RemoveExistingNodes()
        {
            // berprfe, ob die Nodes vorhanden sind und lsche sie gegebenenfalls
            Atom Node_UpLeft = SuperController.singleton.GetAtomByUid("Node_UpLeft");
            if (Node_UpLeft != null)
            {
                SuperController.singleton.RemoveAtom(Node_UpLeft);
            }

            Atom Node_UpRight = SuperController.singleton.GetAtomByUid("Node_UpRight");
            if (Node_UpRight != null)
            {
                SuperController.singleton.RemoveAtom(Node_UpRight);
            }

            Atom Node_DownRight = SuperController.singleton.GetAtomByUid("Node_DownRight");
            if (Node_DownRight != null)
            {
                SuperController.singleton.RemoveAtom(Node_DownRight);
            }

            Atom Node_DownLeft = SuperController.singleton.GetAtomByUid("Node_DownLeft");
            if (Node_DownLeft != null)
            {
                SuperController.singleton.RemoveAtom(Node_DownLeft);
            }
        }

        void UpdateEmptyNodesPosition()
        {
            // Aktualisiere die Position der Empty Nodes basierend auf den Elternelementen
            if (BDSMPillar_UpLeft != null)
            {
                Atom Node_UpLeft = SuperController.singleton.GetAtomByUid("Node_UpLeft");
                if (Node_UpLeft != null)
                {
                    Node_UpLeft.mainController.transform.position = BDSMPillar_UpLeft.position;
                }
            }

            if (BDSMPillar_UpRight != null)
            {
                Atom Node_UpRight = SuperController.singleton.GetAtomByUid("Node_UpRight");
                if (Node_UpRight != null)
                {
                    Node_UpRight.mainController.transform.position = BDSMPillar_UpRight.position;
                }
            }

            if (BDSMPillar_DownRight != null)
            {
                Atom Node_DownRight = SuperController.singleton.GetAtomByUid("Node_DownRight");
                if (Node_DownRight != null)
                {
                    Node_DownRight.mainController.transform.position = BDSMPillar_DownRight.position;
                }
            }

            if (BDSMPillar_DownLeft != null)
            {
                Atom Node_DownLeft = SuperController.singleton.GetAtomByUid("Node_DownLeft");
                if (Node_DownLeft != null)
                {
                    Node_DownLeft.mainController.transform.position = BDSMPillar_DownLeft.position;
                }
            }
        }


        void LateUpdate()
        {
            // Aktualisiere die Position der Empty Nodes in LateUpdate
            UpdateEmptyNodesPosition();
        }

        

        private float GetSliderValue(float currentValue, float startPosition, float maxValue)
        {
            return Mathf.InverseLerp(startPosition, maxValue, currentValue);
        }

        private Transform getObjectRoot(Transform parent)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(objectNameSearchString))
                {
                    return child;
                }
                else
                {
                    Transform childSearch = getObjectRoot(child);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        void Slider1Changed()
        {
            //SuperController.LogMessage("Slider1Changed");
            float sliderValue = ScrubA.val;

            // Berechne die neue Y-Position basierend auf der Startposition und dem Slider-Wert
            float newY = BDSMPillar_UpLeftStartPosition.y + sliderValue;

            // Setze die neue Position von BDSMPillar_UpLeft unter Beibehaltung der X- und Z-Positionen
            Vector3 newPosition = BDSMPillar_UpLeftStartPosition;
            newPosition.y = newY;
            BDSMPillar_UpLeft.localPosition = newPosition;

            UpdateEmptyNodesPosition();
            previousValue1 = sliderValue;

        }

        void Slider2Changed()
        {
            
            float sliderValue = ScrubB.val;
            

            // Berechne die neue Y-Position basierend auf der Startposition und dem Slider-Wert
            float newY = BDSMPillar_UpRightStartPosition.y + sliderValue;

            // Setze die neue Position von BDSMPillar_UpRight unter Beibehaltung der X- und Z-Positionen
            Vector3 newPosition = BDSMPillar_UpRightStartPosition;
            newPosition.y = newY;
            BDSMPillar_UpRight.localPosition = newPosition;
            
            UpdateEmptyNodesPosition();
            previousValue2 = sliderValue;
        }

        void Slider3Changed()
        {
            
            float sliderValue = ScrubC.val;

            // Berechne die neue Y-Position basierend auf der Startposition und dem Slider-Wert
            float newY = BDSMPillar_DownRightStartPosition.y + sliderValue;

            // Setze die neue Position von BDSMPillar_DownRight unter Beibehaltung der X- und Z-Positionen
            Vector3 newPosition = BDSMPillar_DownRightStartPosition;
            newPosition.y = newY;
            BDSMPillar_DownRight.localPosition = newPosition;
            
            UpdateEmptyNodesPosition();
            previousValue3 = sliderValue;
        }

        void Slider4Changed()
        {
            
            float sliderValue = ScrubD.val;

            // Berechne die neue Y-Position basierend auf der Startposition und dem Slider-Wert
            float newY = BDSMPillar_DownLeftStartPosition.y + sliderValue;

            // Setze die neue Position von BDSMPillar_DownLeft unter Beibehaltung der X- und Z-Positionen
            Vector3 newPosition = BDSMPillar_DownLeftStartPosition;
            newPosition.y = newY;
            BDSMPillar_DownLeft.localPosition = newPosition;
            
            UpdateEmptyNodesPosition();
            previousValue4 = sliderValue;
        }

    }
}